/*
 * Tracing hooks
 *
 * Copyright (C) 2008 Red Hat, Inc.  All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2.
 *
 * Red Hat Author: Roland McGrath.
 *
 * This file defines hook entry points called by core code where user
 * tracing/debugging support might need to do something.  These entry
 * points are called tracehook_*().  Each hook declared below has a
 * detailed comment giving the context (locking et al) from which it is
 * called, and the meaning of its return value (if any).
 */

#ifndef _LINUX_TRACEHOOK_H
#define _LINUX_TRACEHOOK_H	1

#include <linux/sched.h>
#include <linux/security.h>
#include <linux/utrace.h>
struct linux_binprm;

/**
 * tracehook_expect_breakpoints - guess if task memory might be touched
 * @task:		current task, making a new mapping
 *
 * Return nonzero if @task is expected to want breakpoint insertion in
 * its memory at some point.  A zero return is no guarantee it won't
 * be done, but this is a hint that it's known to be likely.
 *
 * May be called with @task->mm->mmap_sem held for writing.
 */
static inline int tracehook_expect_breakpoints(struct task_struct *task)
{
	return (tsk_utrace_flags(task) & UTRACE_EVENT(SIGNAL_CORE));
}

/**
 * tracehook_report_syscall_entry - task is about to attempt a system call
 * @regs:		user register state of current task
 *
 * This will be called if %TIF_SYSCALL_TRACE has been set, when the
 * current task has just entered the kernel for a system call.
 * Full user register state is available here.  Changing the values
 * in @regs can affect the system call number and arguments to be tried.
 * It is safe to block here, preventing the system call from beginning.
 *
 * Called without locks, just after entering kernel mode.
 */
static inline void tracehook_report_syscall_entry(struct pt_regs *regs)
{
	if (tsk_utrace_flags(current) & UTRACE_EVENT(SYSCALL_ENTRY))
		utrace_report_syscall(regs, 0);
}

/**
 * tracehook_report_syscall_exit - task has just finished a system call
 * @regs:		user register state of current task
 * @step:		nonzero if simulating single-step or block-step
 *
 * This will be called if %TIF_SYSCALL_TRACE has been set, when the
 * current task has just finished an attempted system call.  Full
 * user register state is available here.  It is safe to block here,
 * preventing signals from being processed.
 *
 * If @step is nonzero, this report is also in lieu of the normal
 * trap that would follow the system call instruction because
 * user_enable_block_step() or user_enable_single_step() was used.
 * In this case, %TIF_SYSCALL_TRACE might not be set.
 *
 * Called without locks, just before checking for pending signals.
 */
static inline void tracehook_report_syscall_exit(struct pt_regs *regs, int step)
{
	if (step)
		force_sig(SIGTRAP, current);
	if (test_thread_flag(TIF_SYSCALL_TRACE) &&
	    (tsk_utrace_flags(current) & UTRACE_EVENT(SYSCALL_EXIT)))
		utrace_report_syscall(regs, 1);
}

/**
 * tracehook_unsafe_exec - check for exec declared unsafe due to tracing
 * @task:		current task doing exec
 *
 * Return %LSM_UNSAFE_* bits applied to an exec because of tracing.
 *
 * Called with task_lock() held on @task.
 */
static inline int tracehook_unsafe_exec(struct task_struct *task)
{
	int unsafe = 0;
	if (tsk_utrace_flags(task))
		unsafe = utrace_unsafe_exec(task);
	return unsafe;
}

/**
 * tracehook_tracer_task - return the task that is tracing the given task
 * @tsk:		task to consider
 *
 * Returns NULL if noone is tracing @task, or the &struct task_struct
 * pointer to its tracer.
 *
 * Must called under rcu_read_lock().  The pointer returned might be kept
 * live only by RCU.  During exec, this may be called with task_lock()
 * held on @task, still held from when tracehook_unsafe_exec() was called.
 */
static inline struct task_struct *tracehook_tracer_task(struct task_struct *tsk)
{
	if (tsk_utrace_flags(tsk))
		return utrace_tracer_task(tsk);
	return NULL;
}

/**
 * tracehook_report_exec - a successful exec was completed
 * @fmt:		&struct linux_binfmt that performed the exec
 * @bprm:		&struct linux_binprm containing exec details
 * @regs:		user-mode register state
 *
 * An exec just completed, we are shortly going to return to user mode.
 * The freshly initialized register state can be seen and changed in @regs.
 * The name, file and other pointers in @bprm are still on hand to be
 * inspected, but will be freed as soon as this returns.
 *
 * Called with no locks, but with some kernel resources held live
 * and a reference on @fmt->module.
 */
static inline void tracehook_report_exec(struct linux_binfmt *fmt,
					 struct linux_binprm *bprm,
					 struct pt_regs *regs)
{
	if (tsk_utrace_flags(current) & UTRACE_EVENT(EXEC))
		utrace_report_exec(bprm, regs);
}

/**
 * tracehook_report_exit - task has begun to exit
 * @exit_code:		pointer to value destined for @current->exit_code
 *
 * @exit_code points to the value passed to do_exit(), which tracing
 * might change here.  This is almost the first thing in do_exit(),
 * before freeing any resources or setting the %PF_EXITING flag.
 *
 * Called with no locks held.
 */
static inline void tracehook_report_exit(long *exit_code)
{
	if (tsk_utrace_flags(current) & UTRACE_EVENT(EXIT))
		utrace_report_exit(exit_code);
}

/**
 * tracehook_clone_prepare - prepare for new child to be cloned
 * @clone_flags:	%CLONE_* flags from clone/fork/vfork system call
 *
 * This is called before a new user task is to be cloned.
 * Its return value will be passed to tracehook_clone_attach().
 *
 * Called with no locks held.
 */
static inline int tracehook_clone_prepare(unsigned clone_flags)
{
	return 0;
}

/**
 * tracehook_clone_attach - new child created and being attached
 * @child:		new child task
 * @trace:		return value from tracehook_clone_prepare()
 *
 * This is called immediately after adding @child to its parent's children list.
 * The @trace value is that returned by tracehook_clone_prepare().
 *
 * Called with current's siglock and write_lock_irq(&tasklist_lock) held.
 */
static inline void tracehook_clone_attach(struct task_struct *child, int trace)
{
	utrace_init_task(child);
}

/**
 * tracehook_report_clone - in parent, new child is about to start running
 * @trace:		return value from tracehook_clone_prepare()
 * @regs:		parent's user register state
 * @clone_flags:	flags from parent's system call
 * @pid:		new child's PID in the parent's namespace
 * @child:		new child task
 *
 * Called after a child is set up, but before it has been started running.
 * The @trace value is that returned by tracehook_clone_prepare().
 * This is not a good place to block, because the child has not started yet.
 * Suspend the child here if desired, and block in tracehook_clone_complete().
 * This must prevent the child from self-reaping if tracehook_clone_complete()
 * uses the @child pointer; otherwise it might have died and been released by
 * the time tracehook_report_clone_complete() is called.
 *
 * Called with no locks held, but the child cannot run until this returns.
 */
static inline void tracehook_report_clone(int trace, struct pt_regs *regs,
					  unsigned long clone_flags,
					  pid_t pid, struct task_struct *child)
{
	if (tsk_utrace_flags(current) & UTRACE_EVENT(CLONE))
		utrace_report_clone(clone_flags, child);
}

/**
 * tracehook_report_clone_complete - new child is running
 * @trace:		return value from tracehook_clone_prepare()
 * @regs:		parent's user register state
 * @clone_flags:	flags from parent's system call
 * @pid:		new child's PID in the parent's namespace
 * @child:		child task, already running
 *
 * This is called just after the child has started running.  This is
 * just before the clone/fork syscall returns, or blocks for vfork
 * child completion if @clone_flags has the %CLONE_VFORK bit set.
 * The @child pointer may be invalid if a self-reaping child died and
 * tracehook_report_clone() took no action to prevent it from self-reaping.
 *
 * Called with no locks held.
 */
static inline void tracehook_report_clone_complete(int trace,
						   struct pt_regs *regs,
						   unsigned long clone_flags,
						   pid_t pid,
						   struct task_struct *child)
{
	if (tsk_utrace_flags(current) & UTRACE_ACTION_QUIESCE)
		utrace_quiescent(current, NULL);
}

/**
 * tracehook_signal_handler - signal handler setup is complete
 * @sig:		number of signal being delivered
 * @info:		siginfo_t of signal being delivered
 * @ka:			sigaction setting that chose the handler
 * @regs:		user register state
 * @stepping:		nonzero if debugger single-step or block-step in use
 *
 * Called by the arch code after a signal handler has been set up.
 * Register and stack state reflects the user handler about to run.
 * Signal mask changes have already been made.
 *
 * Called without locks, shortly before returning to user mode
 * (or handling more signals).
 */
static inline void tracehook_signal_handler(int sig, siginfo_t *info,
					    const struct k_sigaction *ka,
					    struct pt_regs *regs, int stepping)
{
	struct task_struct *tsk = current;
	if ((tsk_utrace_flags(tsk) & UTRACE_EVENT_SIGNAL_ALL)
	    && (tsk_utrace_flags(tsk) & (UTRACE_ACTION_SINGLESTEP
					 | UTRACE_ACTION_BLOCKSTEP)))
		utrace_signal_handler_singlestep(tsk, regs);
}

/**
 * tracehook_consider_ignored_signal - suppress short-circuit of ignored signal
 * @task:		task receiving the signal
 * @sig:		signal number being sent
 * @handler:		%SIG_IGN or %SIG_DFL
 *
 * Return zero iff tracing doesn't care to examine this ignored signal,
 * so it can short-circuit normal delivery and never even get queued.
 * Either @handler is %SIG_DFL and @sig's default is ignore, or it's %SIG_IGN.
 *
 * Called with @task->sighand->siglock held.
 */
static inline int tracehook_consider_ignored_signal(struct task_struct *task,
						    int sig,
						    void __user *handler)
{
	return (tsk_utrace_flags(task) & UTRACE_EVENT(SIGNAL_IGN));
}

/**
 * tracehook_consider_fatal_signal - suppress special handling of fatal signal
 * @task:		task receiving the signal
 * @sig:		signal number being sent
 * @handler:		%SIG_DFL or %SIG_IGN
 *
 * Return nonzero to prevent special handling of this termination signal.
 * Normally @handler is %SIG_DFL.  It can be %SIG_IGN if @sig is ignored,
 * in which case force_sig() is about to reset it to %SIG_DFL.
 * When this returns zero, this signal might cause a quick termination
 * that does not give the debugger a chance to intercept the signal.
 *
 * Called with or without @task->sighand->siglock held.
 */
static inline int tracehook_consider_fatal_signal(struct task_struct *task,
						  int sig,
						  void __user *handler)
{
	return (tsk_utrace_flags(task) & (UTRACE_EVENT(SIGNAL_TERM) |
					  UTRACE_EVENT(SIGNAL_CORE)));
}

/**
 * tracehook_induce_sigpending - let tracing force signal_pending() on
 * @task:		task whose signal_pending() state is being recalculated
 *
 * Called when computing @task's signal_pending() flag.  Return nonzero
 * to force the signal_pending() flag on, so that tracehook_get_signal()
 * will be called before the next return to user mode.
 *
 * Called with @task->sighand->siglock held.
 */
static inline int tracehook_induce_sigpending(struct task_struct *task)
{
	return unlikely(tsk_utrace_flags(task) & UTRACE_ACTION_QUIESCE);
}

/**
 * tracehook_get_signal - deliver synthetic signal to traced task
 * @task:		@current
 * @regs:		task_pt_regs(@current)
 * @info:		details of synthetic signal
 * @return_ka:		sigaction for synthetic signal
 *
 * Return zero to check for a real pending signal normally.
 * Return -1 after releasing the siglock to repeat the check.
 * Return a signal number to induce an artifical signal delivery,
 * setting *@info and *@return_ka to specify its details and behavior.
 *
 * The @return_ka->sa_handler value controls the disposition of the
 * signal, no matter the signal number.  For %SIG_DFL, the return value
 * is a representative signal to indicate the behavior (e.g. %SIGTERM
 * for death, %SIGQUIT for core dump, %SIGSTOP for job control stop,
 * %SIGTSTP for stop unless in an orphaned pgrp), but the signal number
 * reported will be @info->si_signo instead.
 *
 * Called with @task->sighand->siglock held, before dequeuing pending signals.
 */
static inline int tracehook_get_signal(struct task_struct *task,
				       struct pt_regs *regs,
				       siginfo_t *info,
				       struct k_sigaction *return_ka)
{
	if (unlikely(tsk_utrace_flags(task)))
		return utrace_get_signal(task, regs, info, return_ka);
	return 0;
}

/**
 * tracehook_finish_stop - entering job control stop
 * @notify:		nonzero if this is the last thread in the group to stop
 *
 * Called when about to stop for job control; we are already in
 * TASK_STOPPED state, about to call schedule.  Return nonzero
 * to generate a %SIGCHLD with %CLD_STOPPED, which is normal if
 * @notify is nonzero.
 *
 * Called with no locks held.
 */
static inline int tracehook_finish_stop(int notify)
{
	if ((tsk_utrace_flags(current) & UTRACE_EVENT(JCTL)) &&
	    utrace_report_jctl(CLD_STOPPED))
		return 0;
	return notify;
}

/**
 * tracehook_notify_death - task is dead, ready to notify parent
 * @task:		@current task now exiting
 * @death_cookie:	value to pass to tracehook_report_death()
 *
 * Return the signal number to send our parent with do_notify_parent(), or
 * zero to send no signal and leave a zombie, or -1 to self-reap right now.
 *
 * Called with write_lock_irq(&tasklist_lock) held.
 */
static inline int tracehook_notify_death(struct task_struct *task,
					 void **death_cookie)
{
	*death_cookie = tsk_utrace_struct(task);
	if (tsk_utrace_flags(task) & UTRACE_ACTION_NOREAP)
		return 0;

	if (task->exit_signal == -1)
		return -1;

	if (thread_group_empty(task))
		return task->exit_signal;

	return 0;
}

/**
 * tracehook_report_death - task is dead and ready to be reaped
 * @task:		@current task now exiting
 * @exit_state:		@current->exit_state value, %EXIT_ZOMBIE or %EXIT_DEAD
 * @death_cookie:	value passed back from tracehook_notify_death()
 *
 * Thread has just become a zombie (@exit_state == %EXIT_ZOMBIE) or is
 * about to self-reap (@exit_state == %EXIT_DEAD).  If normal reaping
 * is not inhibited, task->exit_state might be changing in parallel.
 * The @death_cookie was passed back by tracehook_notify_death().
 *
 * Called without locks.
 */
static inline void tracehook_report_death(struct task_struct *task,
					  int exit_state, void *death_cookie)
{
	smp_mb();
	if (tsk_utrace_flags(task) & (UTRACE_EVENT(DEATH) |
				      UTRACE_EVENT(QUIESCE)))
		utrace_report_death(task, death_cookie);
}

/**
 * tracehook_inhibit_wait - control parent's do_wait on this child
 * @child:		child task being considered by do_wait()
 *
 * Return nonzero if the child's parent should be prevented from
 * seeing its child's status changes in wait calls.
 *
 * Called with @tasklist_lock held for reading.
 */
static inline int tracehook_inhibit_wait(struct task_struct *child)
{
	return (tsk_utrace_flags(child) & UTRACE_ACTION_NOREAP);
}


/*
 * XXX below this point still to be reworked XXX
 */

/*
 * Called after a CLONE_VFORK parent has waited for the child to complete.
 * The clone/vfork system call will return immediately after this.
 * The child pointer may be invalid if a self-reaping child died and
 * tracehook_report_clone took no action to prevent it from self-reaping.
 */
static inline void tracehook_report_vfork_done(struct task_struct *child,
					       pid_t child_pid)
{
	if (tsk_utrace_flags(current) & UTRACE_EVENT(VFORK_DONE))
		utrace_report_vfork_done(child_pid);
}

/*
 * Called from release_task, no locks held.
 * After this, there should be no tracing entanglements.
 */
static inline void tracehook_release_task(struct task_struct *p)
{
	smp_mb();
	if (tsk_utrace_struct(p) != NULL)
		utrace_release_task(p);
}

/*
 * Called from release_task with write_lock_irq(&tasklist_lock) held.
 *
 * This should verify that there is no tracing-related state
 * still affecting the task_struct about to be released.
 */
static inline void tracehook_release_task_locked(struct task_struct *p)
{
	int bad = 0;
	BUG_ON(p->exit_state != EXIT_DEAD);
	if (unlikely(tsk_utrace_struct(p) != NULL)) {
		/*
		 * In a race condition, utrace_attach will temporarily set
		 * it, but then check p->exit_state and clear it.  It does
		 * all this under task_lock, so we take the lock to check
		 * that there is really a bug and not just that known race.
		 */
		task_lock(p);
		bad = unlikely(tsk_utrace_struct(p) != NULL);
		task_unlock(p);
	}
	BUG_ON(bad);
}

/*
 * do_notify_parent_cldstop calls this when it's about to generate a SIGCHLD
 * for a job control stop.  Return nonzero to prevent that signal generation.
 * Called with tasklist_lock held for reading, sometimes with irqs disabled.
 */
static inline int tracehook_notify_cldstop(struct task_struct *tsk,
					   const siginfo_t *info)
{
	return (tsk_utrace_flags(tsk) & UTRACE_ACTION_NOREAP);
}

/*
 * This is called when tracehook_inhibit_wait(p) returned true
 * and a previously delayed group_leader is now eligible for reaping.
 * It's called from release_task, with no locks held, and p is not current.
 */
static inline void tracehook_report_delayed_group_leader(struct task_struct *p)
{
	utrace_report_delayed_group_leader(p);
}

#endif	/* <linux/tracehook.h> */
